@@ -1,19 +1,21 @@
 #ifndef __STAN__AGRAD__MATRIX_HPP__
 #define __STAN__AGRAD__MATRIX_HPP__
 
-#include <stan/agrad/rev/matrix/fill.hpp>
 #include <stan/agrad/rev/matrix/Eigen_NumTraits.hpp>
 #include <stan/agrad/rev/matrix/initialize_variable.hpp>
 #include <stan/agrad/rev/matrix/typedefs.hpp>
 #include <stan/agrad/rev/matrix/to_var.hpp>
+#include <stan/agrad/rev/matrix/value_of.hpp>
 #include <stan/agrad/rev/matrix/squared_dist.hpp>
 #include <stan/agrad/rev/matrix/dot_self.hpp>
 #include <stan/agrad/rev/matrix/dot_product.hpp>
+#include <stan/agrad/rev/matrix/sort.hpp>
 #include <stan/agrad/rev/matrix/sum.hpp>
 #include <stan/agrad/rev/matrix/mdivide_left.hpp>
 #include <stan/agrad/rev/matrix/mdivide_left_spd.hpp>
 #include <stan/agrad/rev/matrix/mdivide_left_tri.hpp>
 #include <stan/agrad/rev/matrix/determinant.hpp>
+#include <stan/agrad/rev/matrix/quad_form.hpp>
 #include <stan/agrad/rev/matrix/ldlt.hpp>
 #include <stan/agrad/rev/matrix/log_determinant.hpp>
 #include <stan/agrad/rev/matrix/log_determinant_spd.hpp>
@@ -21,8 +23,8 @@
 #include <stan/agrad/rev/matrix/multiply.hpp>
 #include <stan/agrad/rev/matrix/multiply_lower_tri_self_transpose.hpp>
 #include <stan/agrad/rev/matrix/tcrossprod.hpp>
+#include <stan/agrad/rev/matrix/trace_quad_form.hpp>
+#include <stan/agrad/rev/matrix/trace_gen_quad_form.hpp>
 #include <stan/agrad/rev/matrix/crossprod.hpp>
-#include <stan/agrad/rev/matrix/assign_to_var.hpp>
-#include <stan/agrad/rev/matrix/assign.hpp>
 
 #endif
