@@ -17,6 +17,7 @@ namespace stan {
      * specified input, parse it, and write the C++ code for it
      * to the specified output.
      *
+     * @param msgs Output stream for warning messages
      * @param stan_gm_in Graphical model specification
      * @param cpp_out C++ code output stream
      * @param model_name Name of model class
@@ -28,14 +29,14 @@ namespace stan {
      *    due to syntax error in the Graphical model; 
      *    <code>true</code> otherwise.
      */
-    bool compile(std::ostream* output_stream, // for warnings
+    bool compile(std::ostream* msgs, // for warnings
                  std::istream& stan_gm_in,
                  std::ostream& cpp_out,
                  const std::string& model_name,
                  bool include_main = true,
                  const std::string& in_file_name = "input") {
       program prog;
-      bool parsed_ok = parse(output_stream,stan_gm_in,in_file_name,prog);
+      bool parsed_ok = parse(msgs,stan_gm_in,in_file_name,model_name,prog);
       if (!parsed_ok) 
         return false; // syntax error in program
       generate_cpp(prog,model_name,cpp_out,include_main);
