@@ -1,23 +1,23 @@
 #ifndef __STAN__MATH__REP_MATRIX_HPP__
 #define __STAN__MATH__REP_MATRIX_HPP__
 
-#include <stdexcept>
-#include <vector>
-
+#include <boost/math/tools/promotion.hpp>
+#include <stan/math/validate_non_negative_rep.hpp>
 #include <stan/math/matrix/Eigen.hpp>
 
-#include <stan/math/rep_array.hpp>
-
 namespace stan {
 
   namespace math {
 
     template <typename T>
-    inline Eigen::Matrix<T,Eigen::Dynamic,Eigen::Dynamic>
+    inline 
+    Eigen::Matrix<typename boost::math::tools::promote_args<T>::type,
+                  Eigen::Dynamic,Eigen::Dynamic>
     rep_matrix(const T& x, int m, int n) {
       validate_non_negative_rep(m,"rep_matrix rows");
       validate_non_negative_rep(n,"rep_matrix cols");
-      return Eigen::Matrix<T,Eigen::Dynamic,Eigen::Dynamic>::Constant(m,n,x);
+      return Eigen::Matrix<typename boost::math::tools::promote_args<T>::type,
+                           Eigen::Dynamic,Eigen::Dynamic>::Constant(m,n,x);
     }
 
     template <typename T>
