@@ -14,17 +14,23 @@ namespace stan {
     public:
       
       diag_e_point(int n, int m): ps_point(n, m),
-                                  mInv(Eigen::VectorXd::Ones(n)) 
-      {};
+                                  mInv(n) {
+        mInv.setOnes();
+      };
       
       Eigen::VectorXd mInv;
       
-      void write_metric(std::ostream& o) {
-        o << "# Diagonal elements of inverse mass matrix:" << std::endl;
-        o << "# " << mInv(0) << std::flush;
+      diag_e_point(const diag_e_point& z): ps_point(z), mInv(z.mInv.size()) {
+        _fast_vector_copy<double>(mInv, z.mInv);
+      }
+      
+      void write_metric(std::ostream* o) {
+        if(!o) return;
+        *o << "# Diagonal elements of inverse mass matrix:" << std::endl;
+        *o << "# " << mInv(0) << std::flush;
         for(size_t i = 1; i < mInv.size(); ++i)
-          o << ", " << mInv(i) << std::flush;
-        o << std::endl;
+          *o << ", " << mInv(i) << std::flush;
+        *o << std::endl;
       };
       
     };
@@ -34,4 +40,4 @@ namespace stan {
 } // stan
 
 
-#endif
\ No newline at end of file
+#endif
